#ifndef __NADS_HEADER_INCLUDED__
#define __NADS_HEADER_INCLUDED__

#if __GNUC__
#define __hidden __attribute__((visibility("hidden")))
#else
#define __hidden
#endif

struct nads {
	const char	*buf;
	char 		*uri;
	char		*query;
	const char	*host;
	u_int16_t	port;
};

#define NADS_MAX_URI			16384

/* return values are one of these */
#define NADS_OK				1
#define NADS_FAIL			0

/* Possible error codes */
#define NADS_ERR_SUCCESS		0
#define NADS_ERR_NO_EMULATION		1
#define NADS_ERR_BAD_HEX		2
#define NADS_ERR_TRAVERSAL		3
#define NADS_ERR_BAD_UTF8		4
#define NADS_ERR_NULL			5
#define NADS_ERR_OOM			6

/* Web-server details */
#define NADS_WS_CASE_SENSITIVE		0x01UL /* Server has case sensitive paths */
#define NADS_WS_MS_UTF16		0x02UL /* MS-UTF-16 (%u1234) */
#define NADS_WS_PERCENT_HACK		0x04UL /* Allow %%35 etc. */
#define NADS_WS_DOUBLE_HEX		0x08UL /* Do hex decode twice */

/* =====[ Exported API ]===== */

int nads_normalize(struct nads *res);
int nads_default_webserver(const char *name);
int nads_webserver(const char *host, u_int16_t port, const char *emu);
int nads_error(void);
const char *nads_strerror(int err);
const char *nads_errstr(void);

/* =====[ Internal API ]===== */

int __hidden nads_ws(unsigned long x);
void __hidden select_webserver(const char *host, u_int16_t port);
extern int __hidden nads_errcode;

#endif /* __NADS_HEADER_INCLUDED__ */
