/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
* This program is released under the terms of the GNU GPL version 2
*/

#ifndef __STRING_HASH_HEADER_INCLUDED__
#define __STRING_HASH_HEADER_INCLUDED__

/* Hash size */
#ifdef STRING_HASH_64
typedef u_int64_t strhash_t;
#define FNV_PRIME	1099511628211ULL
#define FNV_OFFSET	14695981039346656037ULL
#else
typedef u_int32_t strhash_t;
#define FNV_PRIME	16777619UL
#define FNV_OFFSET	2166136261UL
#endif

typedef u_int32_t iphash_t;

/* Fowler/Noll/Vo hash 
* Its fast to compute, its obvious code, it performs well... */
static inline strhash_t string_hash(const char *str)
{
	strhash_t h=FNV_OFFSET;
	u_int8_t *data=(u_int8_t *)str;

	while(*data)
		h = (h * FNV_PRIME) ^ tolower(*data++);

	return h;
}

static inline iphash_t webserver_hash(const char *host, u_int16_t port)
{
	iphash_t h = string_hash(host);
	return (h ^ port);
}

#endif /* __STRING_HASH_HEADER_INCLUDED__ */
