/*
 * This file is part of nads
 * Copyright (c) 2003 ECSC Ltd.
 * Author: Gianni Tedesco <gianni@scaramanga.co.uk>
 * Released under the terms of the GNU GPL version 2
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <nads.h>

/* The actual error code, kinda like errno */
int __hidden nads_errcode;

static const char *estr[]={
	[ NADS_ERR_SUCCESS ] "Success",
	[ NADS_ERR_NO_EMULATION ] "Unable to emulate specified webserver",
	[ NADS_ERR_BAD_HEX ] "Bad Hex encoding",
	[ NADS_ERR_TRAVERSAL ] "Attempt to traverse outside of webroot",
	[ NADS_ERR_BAD_UTF8 ] "Bad UTF-8 character sequence",
	[ NADS_ERR_NULL ] "NUL character insertion",
	[ NADS_ERR_OOM ] "Out of memory",
};

/* nads_error:
 *
 * Get the nads error code.
 *
 * Return Value: The error code for the last call in to the nads
 * library.
 */
int nads_error(void)
{
	return nads_errcode;
}

/* nads_strerror:
 * @err: A nads error code returned from nads_error()
 * 
 * Given a nads error code, return a string describing the error.
 *
 * Return value: A pointer to a string constant containing an
 * error description in english.
 */
const char *nads_strerror(int err)
{
	if ( err > sizeof(estr)/sizeof(*estr) )
		return "Unknown error";

	return estr[err];
}

/* nads_errstr:
 *
 * Returns a string describing the error for the last call in to the
 * nads library.
 *
 * Return value: A pointer to a string constant containing an
 * error description in english.
 */
const char *nads_errstr(void)
{
	return nads_strerror(nads_errcode);
}
