/*
 * This file is part of nads
 * Copyright (c) 2003 ECSC Ltd.
 * Author: Gianni Tedesco <gianni@scaramanga.co.uk>
 * Released under the terms of the GNU GPL version 2
*/

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <nads.h>

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#ifdef HAVE_GETOPT_LONG
static const struct option opts[]={
	{"webserver",	1, 0, 'p'},
	{"help",	0, 0, 'h'},
	{NULL, 0, 0, 0}
};
#endif

/* usage:
 * @cmd: command name we were invoked with
 * @code: exit code
 *
 * Prints the usage string then exits.
 */
static __attribute__((noreturn)) void
usage(const char *cmd, int code)
{
	fprintf(stderr, "Usage: %s [-w webserver] < urls\n", cmd);
	fprintf(stderr, "N.A.D.S HTTP normalization test application\n\n");
	fprintf(stderr, "  --webserver, -w      Select default webserver emulation\n");
	fprintf(stderr, "  --help, -h           This message\n\n");
	exit(code);
}

static void do_line(char *buf)
{
	struct nads n = {.host = NULL, .port = 0};
	char *uri = NULL;

	/* Simple heuristic to grab URIs out of the input */
	for(; *buf; buf++) {
		if ( uri ) {
			if ( isspace(*buf) ) {
				*buf = '\0';
				break;
			}
		}else if ( *buf == '/' ) {
			uri = buf;
		}
	}

	if ( uri == NULL )
		return;
	
	n.buf = uri;

	if ( nads_normalize(&n) == NADS_OK ) {
		if ( n.query ) {
			printf("%s?%s : OK\n", n.uri, n.query);
		}else{
			printf("%s : OK\n", n.uri);
		}
	}else{
		printf("%s : ERR %s\n", n.buf, nads_errstr());
	}
}
int main(int argc, char **argv) 
{
	static char buf[8192];
	const char *cmd = argv[0];
	const char *ws = NULL;
	int c;
	int state=0;
	int line;
	char *end;

#ifdef HAVE_GETOPT_LONG
	while( (c=getopt_long(argc, argv, "w:e:h?",
		opts, NULL)) != -1 )
#else
	while( (c=getopt(argc, argv, "w:e:h?")) != -1 )
#endif
	{
		switch ( c ) {
		case 'w':
			if ( ws )
				usage(cmd, 1);
			ws = optarg;
			break;
		case 'h':
			usage(cmd, 0);
			break;
		case '?':
			usage(cmd, 1);
			break;
		}
	}

	if ( ws && !nads_default_webserver(ws) ) {
		fprintf(stderr, "%s: %s: %s\n", argv[0], ws, nads_errstr());
		return 1;
	}

	for(line=1; fgets(buf, sizeof(buf), stdin);) {
		end = strchr(buf, '\n');

		if ( state == 1 ) {
			if ( end != NULL ) {
				fprintf(stderr, "%s: line %i too long\n",
					argv[0], line);
				state = 0;
				line++;
			}
			continue;
		}else if ( end == NULL ) {
			state = 1;
			continue;
		}

		*end = '\0';
		line++;

		do_line(buf);
	}

	return 0;
}
